/***************************************************
* 			 SD20 Example for PIC18F4410           *
*   Moves all servos between minimum and maximum   *
*             By James Henderson, 2009			   *
***************************************************/

#include <p18cxxx.h>
#include "stdio.h"
#include "stdlib.h"
#include "SD20.h"

// Configuration Bit Values
/** CONFIGURATION **************************************************/
        #pragma config OSC      = INTIO67
        #pragma config FCMEN    = OFF
        #pragma config IESO     = OFF
        #pragma config PWRT     = ON
        #pragma config BOREN    = OFF
        #pragma config WDT      = OFF
        #pragma config WDTPS    = 32768
        #pragma config MCLRE    = OFF
        #pragma config LPT1OSC  = OFF
        #pragma config PBADEN   = OFF
        #pragma config STVREN   = ON
        #pragma config LVP      = OFF
        #pragma config XINST    = OFF       			// Extended Instruction Set
        #pragma config CP0      = OFF
        #pragma config CP1      = OFF
        #pragma config CPB      = OFF
        #pragma config WRT0     = OFF
        #pragma config WRT1     = OFF
        #pragma config WRTB     = OFF       			// Boot Block Write Protection
        #pragma config WRTC     = OFF
        #pragma config EBTR0    = OFF
        #pragma config EBTR1    = OFF
        #pragma config EBTRB    = OFF

void main(void)
{
unsigned char x;

	setup();
	while(1)
	{
		for(x = 1; x < 21; x++)
		{
			i2c_tx(SD20,x,1);		// Servos to minimum position
			delay100uS();			// A short delay before sending the next i2c command to let the SD20 process the information
		}
		SetTimer(37000);			// Dealy 300mS to let the servos move to the new position				
		Wait4Timer();
		for(x = 1; x < 21; x++)
		{
			i2c_tx(SD20,x,255);		// Servos to minimum position
			delay100uS();			// A short delay before sending the next i2c command to let the SD20 process the information
		}
		SetTimer(37000);			// Dealy 300mS to let the servos move to the new position
		Wait4Timer();
	}
}

/*****************
* Timer Routines *
*****************/
void SetTimer(unsigned int time)
{
	TMR0H = 0-(time/256);
	TMR0L = 0-(time&0xff);
	INTCONbits.TMR0IF = 0;
}

void Wait4Timer(void)
{
	while(INTCONbits.TMR0IF==0);
}

void delay100uS(void)
{
unsigned char i;
	
	for(i = 0; i < 36; i++);
}

/*************
* Chip setup *
*************/
void setup(void)
{		
	T0CON = 0x83;				// 16bit, 64:1 prescaler
	ADCON1 = 0x0f;				// all ports to digital

	OSCCON = 0x70;				// set 8MHz
	TRISC = 0x1F;
	SSPSTAT = 0x80;				// slew diabled
	SSPCON1 = 0x38;				// master mode enabled
	SSPADD = 0x11;				// 100khz
}


/********************************
* General comunication routines *
********************************/

void i2c_tx(unsigned char address, unsigned char reg, unsigned char data)
{
	SSPCON2bits.SEN = 1;		// innitiate a start bit
	while(SSPCON2bits.SEN);		// Wait for start bit to end
	PIR1bits.SSPIF = 0;			// Clear the i2c interupt flag
	SSPBUF = address;			// i2c address
	while(!PIR1bits.SSPIF);		// Wait for interupt flag to be set indicating and of a transmission
	PIR1bits.SSPIF = 0;		
	SSPBUF = reg;				// register to send i2c data to
	while(!PIR1bits.SSPIF);		
	PIR1bits.SSPIF = 0;		
	SSPBUF = data;				// data to be sent
	while(!PIR1bits.SSPIF);	
	SSPCON2bits.PEN = 1;		// Send stop bit
	while(SSPCON2bits.PEN);		// Wait for stop bit to finish
}

